/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/





//created by Andrew Caldwell on 10/17/99 caldwell@cs.ucla.edu

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include "baseBlkSplitter.h"
#include "ABKCommon/abkcommon.h"

	//this verson finds the best row to split at
void BaseBlockSplitter::createHSplitBlocks
		(PartitioningProblemForCapo& problem)
{
    const SubHGraph&    hgraph = problem.getSubHGraph();
    const Partitioning&  soln  = problem.getBestSoln();

    double totalCellArea       = _block.getTotalCellArea();
    double p0CellArea = 0;
    double p1CellArea = 0;

    unsigned n;
    for(n = hgraph.getNumTerminals(); n != hgraph.getNumNodes(); n++)
    {
	unsigned part   = soln[n].isInPart(0) ? 0 : 1;

	if(part==0)
		p0CellArea += hgraph.getNodeByIdx(n).getWeight();
	else
		p1CellArea += hgraph.getNodeByIdx(n).getWeight();
    }

    if(p0CellArea+p1CellArea != totalCellArea)
    {
	char errorString[1024];
	sprintf(errorString, "Areas do not sum: %f %f : %f\n",
		p0CellArea, p1CellArea, totalCellArea);
        abkfatal(0, errorString);
    }

    double   capacity    = _block.getCapacity();
    unsigned splitRow    = 1;
    double   p1Cap       = _block.getContainedAreaInCoreRow(0);
    double   p0Cap       = capacity - p1Cap;
    double   p0RelWS     = (p0Cap - p0CellArea)/p0Cap;
    double   p1RelWS     = (p1Cap - p1CellArea)/p1Cap;

    double minRelWSDelta = dbl_abs(p0RelWS - p1RelWS);

    for(unsigned r = 1; r < _block.getNumRows()-1; r++)
    {
        p1Cap += _block.getContainedAreaInCoreRow(r);

	p0Cap = capacity - p1Cap;
	p0RelWS = (p0Cap - p0CellArea)/p0Cap;
	p1RelWS = (p1Cap - p1CellArea)/p1Cap;

	double relWSDelta = dbl_abs(p0RelWS - p1RelWS);

        if(relWSDelta < minRelWSDelta)
        {
            minRelWSDelta = relWSDelta;
            splitRow = r+1;
        }
    }

//  if(_params.verb.forMajStats > 4)
// out<<"Computed splitrow is "<<splitRow<<endl;

    createHSplitBlocks(problem, splitRow);
}

	//this version uses the splitRow it's given
void BaseBlockSplitter::createHSplitBlocks
		(PartitioningProblemForCapo& problem, unsigned splitRow)
{
    vector<double>                newBlockCellAreas(2,0.0);
    vector<vector<unsigned> >     cellsInNewBlocks(2);

    const SubHGraph&    hgraph = problem.getSubHGraph();
    const Partitioning&  soln  = problem.getBestSoln();
    unsigned nId;
    for(nId = hgraph.getNumTerminals(); 
        nId != hgraph.getNumNodes(); nId++)
    {
        unsigned origId = hgraph.newNode2OrigIdx(nId);
        unsigned part = soln[nId].lowestNumPart();

        cellsInNewBlocks[part].push_back(origId);
        newBlockCellAreas[part] += hgraph.getNodeByIdx(nId).getWeight();
    }

    vector<double>         newBlockSiteAreas(2);
   
    for(unsigned r = 0; r < splitRow; r++)
        newBlockSiteAreas[1] += _block.getContainedAreaInCoreRow(r);

    newBlockSiteAreas[0] = _block.getCapacity()-newBlockSiteAreas[1];

    const vector<const RBPCoreRow*>& rows = _block.getRows();

    itCBRowOffset sBegin = _block._startOffsets.begin();
    itCBRowOffset sEnd   = _block._startOffsets.end();
    itCBRowOffset eBegin = _block._endOffsets.begin();
    itCBRowOffset eEnd   = _block._endOffsets.end();

    vector<CROffset>    p0StartOffsets(sBegin+splitRow, sEnd);
    vector<CROffset>    p0EndOffsets  (eBegin+splitRow, eEnd);
    vector<CROffset>    p1StartOffsets(sBegin, sBegin+splitRow);
    vector<CROffset>    p1EndOffsets  (eBegin, eBegin+splitRow);

    double ySplit = rows[splitRow]->getYCoord();

    _block.unLinkNeighbors();  // will link children instead
        // determine neighbors of forthcoming child blocks
    vector<CapoBlock*> p0LeftNeighbors, p1LeftNeighbors;

    unsigned k;
    for(k=0; k!=_block._leftNeighbors.size(); k++)
    {
           CapoBlock* tempB=_block._leftNeighbors[k];
           double yMax=tempB->getBBox().yMax;
           double yMin=tempB->getBBox().yMin;
           if (yMax> ySplit) p0LeftNeighbors.push_back(tempB);
           if (yMin< ySplit) p1LeftNeighbors.push_back(tempB);
    }

    vector<CapoBlock*> p0RightNeighbors, p1RightNeighbors;
    for(k=0; k!=_block._rightNeighbors.size(); k++)
    {
           CapoBlock* tempB=_block._rightNeighbors[k];
           double yMax=tempB->getBBox().yMax;
           double yMin=tempB->getBBox().yMin;
           if (yMax> ySplit) p0RightNeighbors.push_back(tempB);
           if (yMin< ySplit) p1RightNeighbors.push_back(tempB);
    }

    vector<CapoBlock*> noNeighbors; //to prevent duplicate links
                                        //between child blk... see below

    const HGraphWDimensions& nlHGraph = _capo.getNetlistHGraph();

    unsigned oldNameLen = strlen(_block.getName());
    char* blockName = new char[oldNameLen+4];
    strcpy(blockName, _block.getName()); 
    strcat(blockName, "H0_");

    if(cellsInNewBlocks[0].size() > 0)
    {
          _newBlocks.push_back(new CapoBlock( cellsInNewBlocks[0],
                                nlHGraph,
                                _block.rowsBegin()+splitRow, _block.rowsEnd(),
                                p0StartOffsets, p0EndOffsets,
                                _block._neighborsAbove, noNeighbors,
                                p0LeftNeighbors,  p0RightNeighbors,
			        blockName));
          _newBlocks.back()->linkNeighbors();
                         // children get linked in instead of block
    }

    blockName[oldNameLen+1] = '1';

    if(cellsInNewBlocks[1].size() > 0)
    {
	  _newBlocks.push_back(new CapoBlock( cellsInNewBlocks[1],
                                nlHGraph,
                                _block.rowsBegin(), _block.rowsBegin()+splitRow,
                                p1StartOffsets, p1EndOffsets,
                                noNeighbors, _block._neighborsBelow,
                                p1LeftNeighbors,  p1RightNeighbors,
				blockName));
	  _newBlocks.back()->linkNeighbors();
                        // children get linked in instead of block
    }

    if (cellsInNewBlocks[0].size() && cellsInNewBlocks[1].size())
    {
          abkfatal( _newBlocks.size() == 2, "didn't create a block");
          _newBlocks[0]->_addNeighborBelow(_newBlocks[1]);
          _newBlocks[1]->_addNeighborAbove(_newBlocks[0]);
          _newBlocks[0]->_sibling=_newBlocks[1];
          _newBlocks[1]->_sibling=_newBlocks[0];
    }
}


void BaseBlockSplitter::createVSplitBlocks
	(PartitioningProblemForCapo& problem, double xSplit)
{
    vector<double>            partAreas(2,0.0);
    vector<vector<unsigned> > cellsInNewBlocks(2);

    const Partitioning& soln = problem.getBestSoln();
    const SubHGraph&    hgraph = problem.getSubHGraph();

    for(unsigned c = hgraph.getNumTerminals();c < hgraph.getNumNodes(); c++)
    {
        unsigned origId = hgraph.newNode2OrigIdx(c);
        unsigned part = soln[c].isInPart(0) ? 0 : 1;

        cellsInNewBlocks[part].push_back(origId);
        partAreas[part] += hgraph.getNodeByIdx(c).getWeight();
    }

    createVSplitBlocks(cellsInNewBlocks, partAreas, xSplit);
}

void BaseBlockSplitter::createVSplitBlocks
		(const vector<vector<unsigned> >& cellsInNewBlocks)
{
	//we're given the cell Ids (can be assumed to all be in 
	//the block being split) which go in each new block

    vector<double>            partAreas(2,0.0);

    const HGraphWDimensions& netlistHG = _capo.getNetlistHGraph();

    unsigned n;
    for(n = 0; n < cellsInNewBlocks[0].size(); n++)
    {
	unsigned cellId = cellsInNewBlocks[0][n];
        partAreas[0] += netlistHG.getNodeByIdx(cellId).getWeight();
    }

    for(n = 0; n < cellsInNewBlocks[0].size(); n++)
    {
	unsigned cellId = cellsInNewBlocks[1][n];
        partAreas[1] += netlistHG.getNodeByIdx(cellId).getWeight();
    }


    double xSplit = _block.getBBox().xMin;
    xSplit += _block.getBBox().getWidth() *
			(partAreas[0]/partAreas[0]+partAreas[1]);

    createVSplitBlocks(cellsInNewBlocks, partAreas, xSplit);
}

void BaseBlockSplitter::createVSplitBlocks
		(const vector<vector<unsigned> >& cellsInNewBlocks,
		 const vector<double>&            partAreas, 
		 double				  xSplit)
{
    static_cast<void>(partAreas);

    vector<double>            siteAreas(2,0.0);
    _block.computePartAreas(xSplit, siteAreas);

//  cout<<"XSplit: "<<xSplit<<endl;
//  cout<<"Computed part areas: "<<siteAreas<<endl;

    const vector<const RBPCoreRow*>& rows = _block.getRows();

    //construct the vectors of coreRows and [start/end]offsets

    vector<const RBPCoreRow*> p0CoreRows;
    vector<const RBPCoreRow*> p1CoreRows;
    vector<CROffset>          p0StartOffsets;
    vector<CROffset>          p0EndOffsets;
    vector<CROffset>          p1StartOffsets;
    vector<CROffset>          p1EndOffsets;
    BBox                      p0BBox(_block.getBBox());
    BBox                      p1BBox(_block.getBBox());
    p0BBox.xMax = xSplit;
    p1BBox.xMin = xSplit;


    for(unsigned crId = 0; crId < rows.size(); crId++)
    {
            const RBPCoreRow& cr = *(rows[crId]);
            const double spacing   = cr.getSpacing();

            const CROffset&   sOffset = _block._startOffsets[crId];
            const CROffset&   eOffset = _block._endOffsets[crId];
            const RBPSubRow&  leftSR  = cr[sOffset.first];
            const RBPSubRow&  rightSR = cr[eOffset.first];

            double leftSiteEdge  = leftSR.getXStart() + spacing*sOffset.second;
            double rightSiteEdge = rightSR.getXStart()+
                                spacing*(eOffset.second+1);
                //eOffset.second is the last CONTAINED site, so the right
                //edge of it is the left edge of the 1st not contained site...
                //or eOffset.second+1

            if(rightSiteEdge <= xSplit)
            {   //core row is completely inP0
                p0CoreRows.    push_back(rows[crId]);
                p0StartOffsets.push_back(sOffset);
                p0EndOffsets.  push_back(eOffset);
            }
            else if(leftSiteEdge >= xSplit)
            {   //core row is completely in P1
                p1CoreRows.    push_back(rows[crId]);
                p1StartOffsets.push_back(sOffset);
                p1EndOffsets.  push_back(eOffset);
            }
            else
            {   //the coreRow is split
                p0CoreRows.    push_back(rows[crId]);
                p1CoreRows.    push_back(rows[crId]);
                p0StartOffsets.push_back(sOffset);
                p1EndOffsets.  push_back(eOffset);

                unsigned srId;
                for(srId = sOffset.first; srId <= eOffset.first; srId++)
                {
                    if(cr[srId].getXEnd() <= xSplit) continue; //totally in P0

                    if(cr[srId].getXStart() >= xSplit) //1st one totally in P1
                    {
                        p1StartOffsets.push_back(CROffset(srId,0));
                        p0EndOffsets.  push_back(
                                CROffset(srId-1, cr[srId-1].getNumSites()-1));
                        break;
                    }
                    else //this subRow crosses xSplit
                    {
                        CROffset splitOffset(srId, UINT_MAX);
                        p1StartOffsets.push_back(splitOffset);
                        p0EndOffsets.  push_back(splitOffset);

                        double siteOffset = xSplit - cr[srId].getXStart();
                        siteOffset /= spacing;

                        unsigned splitSite =
                                static_cast<unsigned>(rint(siteOffset));
                        p1StartOffsets.back().second = splitSite;
                        p0EndOffsets.back().second   = splitSite-1;
                        break;
                    }
                }
                abkassert(p0StartOffsets.size() == p0EndOffsets.size(),
                        "p0Offset size mismatch");
                abkassert(p1StartOffsets.size() == p1EndOffsets.size(),
                        "p1Offset size mismatch");
            }
    }
    _block.unLinkNeighbors();  // will link children instead

      // determine neighbors of forthcoming child blocks
        vector<CapoBlock*> p0NeighborsAbove, p1NeighborsAbove;
        unsigned k;
        for(k=0; k!=_block._neighborsAbove.size(); k++)
        {
           CapoBlock* tempB=_block._neighborsAbove[k];
           double xMax=tempB->getBBox().xMax;
           double xMin=tempB->getBBox().xMin;
           if (xMin<xSplit) p0NeighborsAbove.push_back(tempB);
           if (xMax>xSplit) p1NeighborsAbove.push_back(tempB);
        }

        vector<CapoBlock*> p0NeighborsBelow, p1NeighborsBelow;
        for(k=0; k!=_block._neighborsBelow.size(); k++)
        {
           CapoBlock* tempB=_block._neighborsBelow[k];
           double xMax=tempB->getBBox().xMax;
           double xMin=tempB->getBBox().xMin;
           if (xMin<xSplit) p0NeighborsBelow.push_back(tempB);
           if (xMax>xSplit) p1NeighborsBelow.push_back(tempB);
        }

        vector<CapoBlock*> noNeighbors;
                        //to prevent duplicate links between child blk
    _newBlocks = vector<CapoBlock*>(2);
    unsigned oldNameLen = strlen(_block.getName());
    char* blockName = new char[oldNameLen+5];
    strcpy(blockName, _block.getName()); 
    strcat(blockName, "V0_");

    const HGraphWDimensions& nlHGraph = _capo.getNetlistHGraph();

    _newBlocks[0] = new CapoBlock(cellsInNewBlocks[0],
                            nlHGraph,
                            p0CoreRows.begin(), p0CoreRows.end(),
                            p0StartOffsets, p0EndOffsets,
                            p0NeighborsAbove, p0NeighborsBelow,
                            _block._leftNeighbors, noNeighbors,
			    blockName);
    blockName[oldNameLen+1] = '1';
    _newBlocks[1] = new CapoBlock(cellsInNewBlocks[1],
                            nlHGraph,
                            p1CoreRows.begin(), p1CoreRows.end(),
                            p1StartOffsets, p1EndOffsets,
                            p1NeighborsAbove, p1NeighborsBelow,
                            noNeighbors, _block._rightNeighbors,
			    blockName);
    _newBlocks[0]->linkNeighbors();
    _newBlocks[1]->linkNeighbors();
    _newBlocks[0]->_addRightNeighbor(_newBlocks[1]);
    _newBlocks[1]->_addLeftNeighbor(_newBlocks[0]);
    _newBlocks[0]->_sibling=_newBlocks[1];
    _newBlocks[1]->_sibling=_newBlocks[0];
// cout<<"done building"<<endl;
}

    //P0 is the top partition
    //capoBlock rows are sorted in INCREASING Y
    //so..P0 will contain rows [_splitRow->(numrows-1)]
    //and P1 rows [0->(_splitRow-1)]
    //(_splitRow, then, gives the number of rows in p1..or,
    //equivlently, the index of the bottom row in P0)

    //WARNING: side-effect! populates actualCaps

unsigned BaseBlockSplitter::findBestSplitRow(const vector<double>& cellAreas,
					     vector<double>& actualCaps)
{
    unsigned splitRow    = UINT_MAX;
    double   p1Cap       = 0;
    double   bestWSDelta = DBL_MAX;
    double   bestP1Cap   = 0;
    double   totalCap    = _block.getCapacity();

    for(unsigned r = 0; r < _block.getNumRows()-1; r++)
    {
               p1Cap  += _block.getContainedAreaInCoreRow(r);
	double p0Cap   = totalCap-p1Cap;
	double relP1WS = (p1Cap - cellAreas[1])/p1Cap;
	double relP0WS = (p0Cap - cellAreas[0])/p0Cap;

	if(dbl_abs(relP0WS - relP1WS) < bestWSDelta)
        {
            bestP1Cap   = p1Cap;
	    bestWSDelta = dbl_abs(relP0WS - relP1WS);
            splitRow    = r+1;
        }
    }

    if(splitRow == 0 || splitRow > _block.getNumRows())
    {
	cout<<endl<<endl;
	cout<<"ERROR: new block has no rows"<<endl;

	cout<<"Cell Areas: "<<cellAreas[0]<<"  "<<cellAreas[1]<<endl;
	cout<<"SplitRow:   "<<splitRow<<endl;
	cout<<_block<<endl;
        abkfatal(splitRow != 0 && splitRow < _block.getNumRows(),
                "new blk has no rows");
    }

    actualCaps = vector<double>(2,0.0);
    actualCaps[0] = _block.getCapacity()-bestP1Cap;
    actualCaps[1] = bestP1Cap;
    return splitRow;
}

