/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




// FMMMclass needs to be #defined in the #including class

template<class Evaluator, class GainContainer>
bool FMMMclass<Evaluator,GainContainer>  ::  pickMoveApplyIt()
{
   if (!pickMove()) return false;  // side effect !
   _gainCo.setPartitionBias(_moveToPart);
   // now apply the move

   updateGains();

   _gainCo.resetAfterGainUpdate();

   PartitionIds& movingIds=(*_curPart)[_movingModuleIdx];

   _moveLog.logMove(_movingModuleIdx,movingIds);

// can be done with movingIds.setToPart(_moveToPart); 
// but that won't support replicated modules 
 {
   movingIds.removeFromPart(_moveFromPart);
   movingIds.addToPart     (_moveToPart);
 }

   _eval.moveModuleTo   (_movingModuleIdx,_moveFromPart,_moveToPart);
   _partLeg.moveModuleTo(_movingModuleIdx,_moveFromPart,_moveToPart);

  abkassert(_gainCo.isConsistent(), "GainCo inconsistent in pickMoveApplyIt");

   return true;
} 

template<class Evaluator, class GainContainer>
bool FMMMclass<Evaluator,GainContainer> :: pickMove()      
/* returns false, if no moves are available */
/* side-effect: stores the chosen move as 3 unsigneds */
/* calls isMoveLegal() and can change gain container, thus not const */
{
    int invalidatedNodesInBucket=0;
    SVGainElement* moveToMake = NULL;
    while((moveToMake = _gainCo.getHighestGainElement()) != NULL)
    {
	_movingModuleIdx = _gainCo.getElementId(*moveToMake);
	_moveToPart      = _gainCo.getElementPartition(*moveToMake);
	_moveFromPart    = (*_curPart)[_movingModuleIdx].lowestNumPart();
              
	if(isMoveLegal() || 
	   (_partLeg.getViolation() == 0 && allowIllegalMoves)) return true;
        else
        {
           if (++invalidatedNodesInBucket < unCorking)
           {
               if (_gainCo.invalidateElement(_moveToPart))
                   invalidatedNodesInBucket=0;
           }
           else
           {
               invalidatedNodesInBucket=0;
	       _gainCo.invalidateBucket(_moveToPart);
           }
        }
  
    }
    return false;//nothing legal to move
}
