/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/





// FMMMclass needs to be #defined in the #including file

template<class Evaluator, class GainContainer>
void FMMMclass<Evaluator,GainContainer>::updateAffectedModules()
// wrt to _movingModuleIdx, _moveFromPart and _moveToPart
{
  unsigned newLockedIds= 1<<_moveToPart;
  const HGFNode& n = _problem.getHGraph().getNodeByIdx(_movingModuleIdx);
  for(itHGFEdgeLocal  e = n.edgesBegin(); e != n.edgesEnd(); e++)
  {
     const HGFEdge& net=(**e);
     unsigned netId=net.getIndex(); 

     if (_lockedNets[netId]) continue;
     if (_lockedNetConfigIds[netId] & (~newLockedIds)) _lockedNets[netId]=true;
     else _lockedNetConfigIds[netId] |=  newLockedIds; 

     if(_eval.netCostNotAffectedByMove(netId,_moveFromPart,_moveToPart)) 
     continue;

     unsigned oldNetCost = _eval.getNetCost(netId);
 
     for(itHGFNodeLocal gn=net.nodesBegin(); gn!=net.nodesEnd(); gn++)
     {
        unsigned gainingModuleIdx= (*gn)->getIndex();
        if (_lockedModules[gainingModuleIdx]) continue; 
        if (gainingModuleIdx==_movingModuleIdx) continue;//will be locked below

        unsigned gnCurPart=(*_curPart)[gainingModuleIdx].lowestNumPart();
        for(unsigned k=0; k!=_numParts; k++)
        {
           if (k==gnCurPart)  continue;
           _eval.updateNetForMovedModule(netId,_moveFromPart,_moveToPart);
           unsigned deltaNetCost = oldNetCost -_eval.computeCostOfOneNet(netId);

           _eval.updateNetForMovedModule(netId,gnCurPart,k);
           unsigned newWhatIfCost= _eval.computeCostOfOneNet(netId);
           _eval.updateNetForMovedModule(netId,_moveToPart,_moveFromPart);
           unsigned deltaWhatIfCost 
                       = _eval.computeCostOfOneNet(netId) - newWhatIfCost;
	   if(deltaWhatIfCost-deltaNetCost != 0)
               updateGainOfModule(gainingModuleIdx, UINT_MAX, k,
					deltaWhatIfCost-deltaNetCost);
           _eval.updateNetForMovedModule(netId,k,gnCurPart);
        }
     }
   }
   // at this point the total cost should be consistent in _eval
   // all nodes at old locations
}
