/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




//created on 09/15/98 by Igor Markov from an fmMoveManager.h
//by Andrew Caldwell and Igor Markov (06/02/98)

#ifndef _FM_MOVE_MANAGER_BASE_H_  
#define _FM_MOVE_MANAGER_BASE_H_

#include <vector>
  using std::vector;
#include "Partitioners/mmTemplBase.h"
#include "PartLegality/1balanceChk.h"
#include "Partitioners/svMoveLog.h"

template<class Evaluator, class GainContainer>
class FMMMbase : public MoveManagerTemplateBase<Evaluator, GainContainer>
{
 protected:

    bit_vector                  _lockedModules; // use it through methods !
    bit_vector                  _lockedNets   ; // use it through methods !
    vector<unsigned>            _lockedNetConfigIds;

    unsigned			_numMovable;
    unsigned			_numTerminals;
    unsigned		 	_numParts;	

    SVMoveLog                   _moveLog;

    unsigned                    _movingModuleIdx;
    unsigned 			_moveFromPart;
    unsigned			_moveToPart;
    unsigned			_numIllegalMoves;

    double			_tol;  //currently, used when populating
					//the GainCo.  Nodes of weight >=
					//_tol are not added to it (they
					//are treated as fixed)
    bool			_allowCorkingNodes;

//          virtual bool  pickMove()=0;          
    /* false if a move can not be chosen, true otherwise */
    /* side effect: stores the result as 3 unsigneds */
    /* calls isMoveLegal() and can change gain container, thus not const */

    inline          bool  isMoveLegal() const /* verifies all PartLegalities */
   { return  _partLeg.isLegalMove(_movingModuleIdx,_moveFromPart,_moveToPart);}

//          virtual 
            void  initializeGainBuckets();

            void  computeGainsOfModule(unsigned moduleIdx);
//          virtual void  computeGains()=0;  // using the above

//  inline virtual void   updateGainOfModule
//         (unsigned moduleIdx, unsigned from, unsigned to, int deltaGain)=0;

//         virtual void   updateAffectedModules()=0;
//         virtual void   updateGains()=0

                   void   setupClip() {  _gainCo.setupClip(); }

    inline         void   lockModule(unsigned moduleIdx)  
    {   _gainCo.removeAllNodesElements(moduleIdx);
        _lockedModules[moduleIdx]=true;
    }

  public:

    FMMMbase(const PartitioningProblem& problem, bool allowCorkingNodes=false);

    virtual bool     pickMoveApplyIt()=0;  
		/* calls pickMove(), updateGains(),
			    _moveLog.logMove() and updates *_curPart */

    virtual void     reinitialize();  
    virtual void     resetTo(PartitioningSolution& newSol);
            void     undo(unsigned count) {_moveLog.undo(count); }
    const SVMoveLog& getMoveLog() const   { return _moveLog;     }
};

template<class Evaluator, class GainContainer>
inline ostream& operator<<(ostream& out,
 const FMMMbase<Evaluator,GainContainer>& fmm)
{ fmm.prettyPrint(out); return out;}

#endif
