/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




#include "abkcommon.h"
#include "partEvals.h"
#include "fmPart.h"

int main(int argc, const char *argv[])
{
  StringParam  baseFileName("f",argc,argv);
  UnsignedParam numStarts  ("num",argc,argv);
  BoolParam     noClip     ("noClip",argc,argv);

  abkfatal(baseFileName.found()," -f baseFileName not found");
  PartitioningProblem problem(baseFileName);
  Partitioning& fixedCo=const_cast<Partitioning&>(problem.getFixedConstr());

  problem.propagateTerminals();
 
  if(numStarts.found()) problem.reserveBuffers(numStarts);

  PartitionerParams params(argc,argv);
 
  cout<<params<<endl;

//FMPartitioner fm(problem, params);

  const unsigned numValues=6;
//const  double  percentTerms[numValues]= 
//                                    {0.20, 0.18, 0.16, 0.14, 0.12, 0.10};
  const  double  percentTerms[numValues]= 
                                      {0.001,0.005,0.01,0.02,0.05,0.10};

  char buffer[200];
  strncpy(buffer,baseFileName,199);
  const char *base=BaseFileName(buffer);
 
  HGraphFixed& hg=const_cast<HGraphFixed&>(problem.getHGraph());
  unsigned numTerm=hg.getNumTerminals();

  abkfatal(hg.getNumNodes()<10000*numTerm," Less than 0.01% terminals");

//Partitioning& fixedCo=const_cast<Partitioning&>(problem.getFixedConstr());
  Partitioning  fixedCoSave=fixedCo;
  vector<unsigned>& term2Block
               =const_cast< vector<unsigned>& >(problem.getTerminalToBlock());
  fill(term2Block.begin(),term2Block.end(),UINT_MAX);

  PartitionIds freeToMove;
  freeToMove.setToAll(2);

  fill(fixedCo.begin(), fixedCo.end(), freeToMove);

  char filename[100], suffix[100];
  strcpy(filename,base);
  sprintf(suffix,"-randFix+%2.2f%%",0.0);
  strcat(filename,suffix);
  cout << filename << endl;
  problem.save(filename); 
 
  vector<unsigned> termIdcs(hg.getNumTerminals());
  iota(termIdcs.begin(), termIdcs.end(), 0);
  random_shuffle(termIdcs.begin(), termIdcs.end());

  {
     unsigned numFixedTerms=numTerm;
     strcpy(filename,base);
     sprintf(suffix,"-randFix+allTerm");
     strcat(filename,suffix);
     cout << filename << endl;

     fill(fixedCo.begin(), fixedCo.end(), freeToMove);
     for(unsigned i=0; i!=numFixedTerms; i++)
       fixedCo[termIdcs[i]]=fixedCoSave[termIdcs[i]];

     problem.save(filename); 
  }

  for(unsigned k=0; k!=numValues; k++)
  {
     unsigned numFixedTerms=hg.getNumNodes()*percentTerms[k];
     if (numFixedTerms==0 || numFixedTerms>numTerm) continue;
     strcpy(filename,base);
     sprintf(suffix,"-randFix+%2.2f%%",percentTerms[k]*100);
     strcat(filename,suffix);
     cout << filename << endl;

     fill(fixedCo.begin(), fixedCo.end(), freeToMove);
     for(unsigned i=0; i!=numFixedTerms; i++)
       fixedCo[termIdcs[i]]=fixedCoSave[termIdcs[i]];

     problem.save(filename); 
  }
  return 0;
}

