/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




//created by Andrew Caldwell and Igor Markov 05/29/98

#ifndef __GAIN_CONTAINER_XFACE_
#define __GAIN_CONTAINER_XFACE_

#include <iostream.h>

template<class GainElement>
class GainContainerInterface
{
  public:

    GainContainerInterface() {}
    virtual ~GainContainerInterface() {}
  
    virtual void reinitialize() = 0;	

    virtual void updateGain(GainElement& elmt, int deltaGain) = 0;
    virtual void updateGainOffset(GainElement& elmt, int deltaGain) = 0;
//  virtual void    setGain(GainElement& elmt, int deltaGain) = 0;
//  virtual void    setGainOffset(GainElement& elmt, int deltaGain) = 0;
    virtual void moveGainToOffset(GainElement& elmt)                = 0;

	//remove an element from the prioritizer
    virtual void removeElement(GainElement& elmt) = 0;
    virtual void removeElement(unsigned nodeId, unsigned part) = 0;
    virtual void removeAllNodesElements(unsigned nodeId) = 0;

	//add an element to the prioritizer
    virtual void addElement(GainElement& elmt, int gain) = 0;
	//derived classes need to add methods to GET a GainElement
	//and also some to add/remove given identifying characteristics
	//of the move (i.e. nodeId & partition for SingleVertex moves, etc)
 
	//returns NULL if there are no more valid moves to make
    virtual GainElement* getHighestGainElement() = 0;

    virtual bool invalidateElement(GainElement& elmt) = 0;
    virtual bool invalidateBucket (GainElement& elmt) = 0;

    virtual void resetAfterGainUpdate() = 0;

};

#endif

