/**************************************************************************
***    
***    Copyright (c) 1995-1999 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA, 
***    and by the Regents of the University of California.    
***    All rights reserved.   
***
***    No portion of this package may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/

// Created by Mike Oliver on 28 april 1999

#include "bbPart.h"
#include <discretePrioritizer.h>
void BBPart::_reorderMovables()
    {
    vector<unsigned> priorities(_movables.size(),0);
    unsigned k;

    //LOOK! make this a member?
    vector<unsigned> mapBack(_hgraph.getNumNodes(),UINT_MAX);
    
    for (k=0;k<_movables.size();k++)
        {
        mapBack[_movables[k]]=k;
        }

    // find degrees of nodes wrt subgraph of movables
    // these will be initial priorities
    for (k=0;k<_movables.size();k++)
        {
        HGFNode &node = _hgraph.getNodeByIdx(_movables[k]);
        itHGFEdgeLocal iE = node.edgesBegin();
        for (;iE!=node.edgesEnd();iE++)
            {
            bool edgeUsed=false;
            HGFEdge &edge = *(*iE);
            itHGFNodeLocal iN = edge.nodesBegin();
            for (;iN!=edge.nodesEnd();iN++)
                {
                HGFNode &otherNode = *(*iN);
                unsigned otherIdx = mapBack[otherNode.getIndex()];
                if (otherIdx != UINT_MAX && otherIdx != k)
                    {
                    edgeUsed=true;
                    break;
                    }
                }
            if (edgeUsed)
                priorities[k] += unsigned(edge.getWeight()+0.5);
            }
        }

    DiscretePrioritizer prior(priorities);

    BitBoard nodeHit(_movables.size());
    BitBoard nodeHit2(_movables.size());
    vector<unsigned> nbrs;  // immediate neighbors
    vector<unsigned> nbrs2; // neighbors of neighbors
    vector<unsigned> importantNodes; // will be at head of _movables
    vector<unsigned> sillyNodes;     // will be at end of _movables

    nbrs.reserve(_movables.size());
    nbrs2.reserve(_movables.size());

    unsigned topIdx;

    while ( (topIdx=prior.top()) != UINT_MAX)
        {
        HGFNode const &topNode = _hgraph.getNodeByIdx(topIdx);

        // highest-priority node removed from prioritizer and
        // added to importantNodes
        importantNodes.push_back(topIdx);
        prior.erase(topIdx);
        nodeHit.setBit(topIdx);

        nbrs.clear();
        itHGFEdgeLocal iE=topNode.edgesBegin();
        for (;iE!=topNode.edgesEnd();iE++)
            {
            HGFEdge const &edge = *(*iE);
            itHGFNodeLocal iN = edge.nodesBegin();
            for (;iN!=edge.nodesEnd();iN++)
                {
                HGFNode const &nbrNode = *(*iN);
                unsigned nbrIdx = mapBack[nbrNode.getIndex()];
                if (nbrIdx != UINT_MAX)
                    {
                    if (!nodeHit.isBitSet(nbrIdx))
                        {
                        nodeHit.setBit(nbrIdx);
                        nbrs.push_back(nbrIdx);
                        }
                    }
                }
            }

        nbrs2.clear();
        nodeHit2.clear();

        // now we've found the neighbors of the important
        // node.  We have to take them off the prioritizer
        // and put them on sillyNodes, and we have to reprioritize
        // *their* neighbors.
        for (unsigned i=0;i<nbrs.size();i++)
            {
            unsigned nbrIdx = nbrs[i];
            sillyNodes.push_back(nbrIdx);
            prior.erase(nbrIdx);
            HGFNode const &nbrNode = _hgraph.getNodeByIdx(_movables[nbrIdx]);

            itHGFEdgeLocal iE = nbrNode.edgesBegin();
            for (;iE!=nbrNode.edgesEnd();iE++)
                {
                HGFEdge const &edge=*(*iE);
                itHGFNodeLocal iN = edge.nodesBegin();
                for (;iN!=edge.nodesEnd();iN++)
                    {
                    HGFNode &secondNode = *(*iN);
                    unsigned secondIdx = mapBack[secondNode.getIndex()];
                    if (secondIdx != UINT_MAX && !nodeHit.isBitSet(secondIdx))
                        {
                        if (!nodeHit2.isBitSet(secondIdx))
                            {
                            nodeHit2.setBit(secondIdx);
                            nbrs2.push_back(secondIdx);
                            }
                        priorities[secondIdx] -=
                            unsigned(edge.getWeight()+0.5);
                        }
                    }
                }
            }
        for (unsigned j=0;j<nbrs2.size();j++)
            {
            unsigned secondIdx = nbrs2[j];
            prior.requeueToHead(secondIdx,priorities[secondIdx]);
            }
        }

    abkfatal(importantNodes.size()+sillyNodes.size()==_movables.size(),
        "Vertex reordering changed "
        "number of vertices");

    vector<unsigned> temp(_movables);
    _movables.clear();
    _movables.reserve(temp.size());

    for (k=0;k<importantNodes.size();k++)
        _movables.push_back(temp[importantNodes[k]]);

    for (k=0;k<sillyNodes.size();k++)
        _movables.push_back(temp[sillyNodes[k]]);

    _setWeights(); //has to be redone because we've changed the order
                   //of the vertices.  LOOK!  can we eliminate some
                   //redundancy here?

    }


